from brancher.branches import LocalBranch, RemoteBranch
from brancher.submission import Submission
from brancher.tarball import Tarball
from brancher.ppa import PPA
from brancher import web

from datetime import datetime

class App(object):
    submission_timestamp = None
    lp = None
    def __init__(self, app_url=None, name=None, submission_url=None, 
                 local_file_sha1sum=None, local_file_timestamp=None, 
                 local_file=None, local_branch_path=None, 
                 available_branches=[], available_ppas=[]):
        self.app_url = app_url
        self.name = name
        self.submission_url = submission_url
        self.local_file_timestamp = local_file_timestamp
        self.local_file_sha1sum = local_file_sha1sum
        self.local_file = local_file
        self.local_branch_path = local_branch_path
        self.set_submission_timestamp(local_file_timestamp)
        self.available_branches = available_branches
        self.available_ppas = available_ppas
        self.log = None

    def set_submission_timestamp(self, submission_timestamp=None):
        if submission_timestamp:
            if isinstance(submission_timestamp, (str, unicode)):
                self.submission_timestamp = datetime.strptime(submission_timestamp, "%Y-%m-%d %H:%M:%S")
            if isinstance(submission_timestamp, datetime):
                self.submission_timestamp = submission_timestamp
        else:
            self.submission_timestamp = submission_timestamp

    def reset(self):
        self.submission_url= None
        self.submission_sha = None
        self.set_submission_timestamp(None)

    def set_log(self, log):
        self.log = log

    def get_web_info(self):
        (self.name, self.submission_url) = web.get_app_data(self.app_url)
        if not self.submission_url:
            self.log.add(error="No tarball link found on: '%s'." % self.app_url)
            return False
        if not self.name:
            self.log.add(error="No app name found on: '%s'." % self.app_url)
            return False
        return True

    def find_available_branches(self):
        sock = web.connect(self.app_url+"feedback/")
        if not sock:
            return []
        html = sock.read()
        sock.close()
        self.available_branches = web.find_branch_info(html)
    
    def find_available_ppas(self, text=None):
        if not text:
            sock = web.connect(self.app_url+"feedback/")
            if not sock:
                return []
            text = sock.read()
            sock.close()
        self.available_ppas = [PPA(url, self.lp, self.name, self.log) for url in \
                                    web.find_ppa_info(text)]
 
    def get_info(self, lp):
        self.lp = lp
        if not self.get_web_info():
            return False
        self.find_available_branches()
        self.find_available_ppas()
        return True

    def push_to_lp(self):
        remote_branch = "lp:~ubuntu-app-review-contributors/ubuntu-app-reviews/%s" % self.name
        self.local_branch.push_to_launchpad(remote_branch)
        if self.log.success:
            self.remote_branch = remote_branch

    def import_branch(self):
        # at some stage we might want to be a bit cleverer than
        # just picking the last branch mentioned in the conversation
        remote_branch = RemoteBranch(self.available_branches[-1])
        self.local_branch.import_remote_branch(remote_branch)

    def import_ppa(self):
        ppa = self.available_ppas[-1]
        ppa.download_package(self.name)
        if self.log.success:
            self.local_branch.import_dsc(ppa.dsc_file())

    def import_code(self):
        self.local_branch = LocalBranch(self.name, self.log)
        if self.available_branches:
            self.import_branch()
        elif self.available_ppas:
            self.import_ppa()
        else:
            submission = Submission(self.submission_url, self.log, 
                                    self.local_file_sha1sum, 
                                    self.local_file_timestamp)
            if submission.needs_import():
                self.local_file = submission.local_file
                self.local_file_timestamp = submission.local_file_timestamp
                self.local_file_sha1sum = submission.local_file_sha1sum
                if submission.is_tarball():
                    tarball = Tarball(self.name, self.local_file, self.log)
                    if tarball.dsc_file:
                        self.local_branch.import_dsc(tarball.local_file)
                    else:
                        self.local_branch.import_tarball(tarball.local_file)
                elif submission.is_text_file():
                    with open(self.local_file, "r") as f:
                        text = f.read()
                    self.available_branches = web.find_branch_info(text)
                    if self.available_branches:
                        self.import_branch()
                    else:
                        self.available_ppas = [PPA(url, self.lp, self.name, self.log) for url in \
                                                web.find_ppa_info(text)]
                        if self.available_ppas:
                            self.import_ppa()
                else:
                    self.log.add(error="No idea what to do with '%s'. " % self.local_file)
        if self.log.success:
            self.local_branch.run_pkgme_if_necessary()
            self.push_to_lp()
