from brancher.app import App
from brancher.job import Log
from brancher import defs

from launchpadlib.launchpad import Launchpad

from datetime import datetime
import codecs
import json
import os

class LP(object):
    conn = None

    def login(self):
        if not self.conn:
            self.conn = Launchpad.login_with("arb-lint", "production",
                            credentials_file=os.path.expanduser("~/.launchpadlib/apps-brancher.cred"))
        return self.conn

class AppCache(object):
    cache = []
    data_file = os.path.join(defs.get_path('cache'), 'cache')
    lp = LP()

    def __init__(self):
        data = []
        if os.path.exists(self.data_file):
            data = json.loads(open(self.data_file).read())
        for entry in data:
            app = App()
            app.__dict__ = entry
            app.set_submission_timestamp(entry['local_file_timestamp'])
            self.cache += [ app ]

    def save(self):
        data = []
        for app in self.cache:
            d = dict(app.__dict__)
            for key in [ 'available_ppas', 'lp', 'log', 'local_branch' ]:
                if d.has_key(key):
                    d.pop(key)
            if d.has_key('local_file_timestamp') and \
               isinstance(d['local_file_timestamp'], datetime):
                d['local_file_timestamp'] = app.tarball_timestamp.strftime("%Y-%m-%d %H:%M:%S")
            data += [ d ]
        if os.path.exists(self.data_file):
            os.remove(self.data_file)
        with codecs.open(self.data_file, 'w', encoding='utf-8') as f:
            f.write(json.dumps(data, ensure_ascii=False))

    def add(self, url):
        found = filter(lambda a: a.app_url == url, self.cache)
        if found:
            return found[0]
        app = App(url)
        self.cache += [ app ]
        return app

    def import_app(self, app_url, retry=False):
        if not app_url.endswith("/"):
            app_url += "/"
        app = self.add(app_url)
        if retry:
            app.reset()
        app.set_log(Log())
        app.log.add(info=''.ljust(60, '-'))
        app.log.add(info="[%s] %s: '%s'." % \
                            (datetime.now().strftime("%F %T"), \
                            "Retrying" if retry else "Processing", \
                            app_url))
        if not app.get_info(self.lp):
            return app.log
        app.log.add(info="Name: '%s', tarball: '%s', Branches: %s, PPAs: %s." % \
                        (app.name, app.submission_url, app.available_branches, 
                         [a.ppa for a in app.available_ppas]))
        app.import_code()
        if app.log.success:
            self.save()
        return app.log
