from job import Action, LoggedActions
from . import defs

import os

class LocalBranch(object):
    def __init__(self, app_name, log):
        self.app_name = app_name
        self.log = log
        self.directory = os.path.join(defs.get_path('branches'), self.app_name)

    def wipe(self):
        if os.path.exists(self.directory):
            Action(["rm", "-r", self.directory]).run()
            return True
        return False

    def import_remote_branch(self, remote_branch):
        # XXX: if remote_branch == self.branch_parent: run('bzr pull')
        self.wipe()
        actions = LoggedActions()
        actions.add(Action(["bzr", "branch", remote_branch.location, self.directory],
                           expected_retcode=0))
        self.log.add(logged_actions=actions)

    def import_dsc(self, dsc_path):
        self.wipe()
        os.makedirs(self.directory)
        actions = LoggedActions()
        if actions.add(Action(["bzr", "init"], self.directory, 0)):
            actions.add(Action(["bzr", "import-dsc", "-v", dsc_path], 
                               self.directory, 0))
        self.log.add(logged_actions=actions)

    def import_tarball(self, tarball):
        self.wipe()
        actions = LoggedActions()
        actions.add(Action(["bzr", "import", "-v", tarball, self.directory],
                           expected_retcode=0))
        self.log.add(logged_actions=actions)

    def push_to_launchpad(self, remote_branch):
        actions = LoggedActions()
        actions.add(Action(["bzr", "push", "--overwrite", remote_branch], 
                           self.directory, 0))
        self.log.add(logged_actions=actions)

    def run_pkgme_if_necessary(self):
        debian_dir = os.path.join(self.directory, "debian")
        if os.path.isdir(debian_dir):
            return
        actions = LoggedActions()
        actions.add(Action(["pkgme", "--nosign", "--nobuild"], 
                           self.directory, 0))
        actions.add(Action(["bzr", "add", "debian"], self.directory, 0))
        actions.add(Action(["bzr", "commit", "-m", 
                            "provide initial packaging by pkgme"],
                            self.directory, 0))
        self.log.add(logged_actions=actions)
        # as the outcome of this action is not strictly required, we can
        # assume success
        self.log.success = True


class RemoteBranch(object):
    def __init__(self, location):
        self.location = location
