import subprocess
import os

from . import defs

class Action(object):
    def __init__(self, command, local_dir=None, expected_retcode=None):
        self.command = command
        self.local_dir = local_dir
        self.stdout = ""
        self.stderr = ""
        self.retcode = 0
        self.success = False
        self.expected_retcode = expected_retcode

    def run(self):
        if self.local_dir:
            pwd = os.getcwd()
            os.chdir(self.local_dir)
        call = subprocess.Popen(self.command, stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)
        self.stdout = call.stdout.read()
        self.stderr = call.stderr.read()
        call.communicate()
        self.retcode = call.returncode
        if self.local_dir:
            os.chdir(pwd)
        if self.expected_retcode is None:
            self.success = True
        elif self.retcode == self.expected_retcode:
            self.success = True

class LoggedActions(object):
    def __init__(self):
        self.log = ""
        self.success = False
        self.actions = []
        
    def add(self, action):
        if not self.actions or self.success:
            # only run if no previous action failed
            self.actions += [ action ]
            action.run()
            self.log += action.stdout
            self.log += action.stderr
            self.success = action.success
        return action.success

class Log(object):
    def __init__(self):
        self.log = ""
        self.success = False

    def add(self, logged_actions=None, info=None, error=None):
        if logged_actions:
            self.log += logged_actions.log
            self.success = logged_actions.success
        if info:
            self.log += (info + "\n")
            self.success = True
        if error:
            self.log += (error + "\n")
            self.success = False
        return self.success

    def write_text_log(self):
        log_file = os.path.join(defs.get_path('log'), 'log.txt')
        text = (self.log + '\n')
        if os.path.exists(log_file):
            with open(log_file, 'r') as f:
                text += f.read()
            os.remove(log_file)
        with open(log_file, 'w') as f:
            f.write(text)
