from . import web
from . import defs

import re
import os

class PPA(object):
    person_name = ""
    ppa_name = ""
    lp = None
    ppa = None
    source = None
    source_package = []

    def __init__(self, ppa_url, lp, app_name, log):
        self.app_name = app_name
        self.log = log
        for regex in [ "^https://launchpad.net/", "lp:" ]:
            ppa_url = re.sub(regex, "", ppa_url)
        found = re.findall(r"^~(.+?)/\+archive\/(.+?)$", ppa_url)
        if found:
            self.person_name = found[0][0]
            self.ppa_name = found[0][1]
        else:
            found = re.findall(r"ppa:(.+?)/(.+?)$", ppa_url)
            if found:
                self.person_name = found[0][0]
                self.ppa_name = found[0][1]
        if self.person_name:
            if not lp.conn:
                lp.login()
            self.lp = lp.conn
            person = self.lp.people[self.person_name]
            found = filter(lambda a: a.name == self.ppa_name, person.ppas)
            if found:
                self.ppa = found[0]
        if not self.ppa:
            self.log.add(error="PPA '%s' not found." % (ppa_url))
            return

    def _find_newest_release(self, sources):
        active_ubuntu_releases = filter(lambda a: a.active, 
                                    self.lp.distributions["ubuntu"].series)
        for release in active_ubuntu_releases:
            for source in sources:
                if release.name == source.distro_series.name:
                    return source
        return None

    def download_package(self, package_name):
        source_packages = self.ppa.getPublishedSources(source_name=package_name, status="Published")
        if source_packages:
            self.source = self._find_newest_release(source_packages)
            if not self.source:
                self.log.add(error="Package '%s' not found in PPA '%s'." % (package_name, self.ppa_name))
                return
            urls = self.source.sourceFileUrls()
            location = os.path.join(defs.get_path("tarballs"), self.app_name)
            if not os.path.isdir(location):
                os.makedirs(location)
            for url in urls:
                file_name = web.retrieve_file(url, location)
                self.source_package.append(file_name)

    def dsc_file(self):
        dsc_files = filter(lambda a: a.endswith(".dsc"), self.source_package)
        if not dsc_files:
            return None
        return dsc_files[0]
