from Crypto.Hash import SHA
import mimetypes

from . import web
from . import defs

class Submission(object):
    remote_timestamp = None
    local_file = None
    def __init__(self, remote_file, log, local_file_sha1sum=None,
                 local_file_timestamp=None):
        self.remote_file = remote_file
        self.log = log
        self.local_file_sha1sum = local_file_sha1sum
        self.local_file_timestamp = local_file_timestamp

    def compute_shasum(self):
        with open(self.local_file) as f:
            sha = SHA.new(f.read())
            return sha.hexdigest()

    def download_submission(self):
        self.local_tarball_path = defs.get_path('tarballs')
        self.local_file = web.retrieve_file(self.remote_file, self.local_tarball_path)
        if not self.local_file:
            self.log.add(error="No tarball found at '%s' or download failed." % self.tarball_url)

    def needs_import(self):
        sock, last_modified = web.get_last_modified(self.remote_file, self.local_file_timestamp)
        if not sock:
            self.log.add(error="Either no tarball found, or no update required.")
            return False
        self.remote_timestamp = last_modified
        sock.close()
        self.download_submission()
        if not self.local_file:
            return False
        new_sha = self.compute_shasum()
        if new_sha != self.local_file_sha1sum:
            self.local_file_sha1sum = new_sha
            self.log.add(info="New SHA. Update necessary.")
            return True
        self.log.add(error="Same SHA. Update not necessary.")
        return False

    def is_tarball(self):
        return filter(lambda suffix: self.local_file.endswith(suffix), 
                      defs.tarball_suffixes)

    def is_text_file(self):
        return mimetypes.guess_type(self.local_file)[0] in [ "text/plain" ]
