from job import Action, LoggedActions
from . import defs

import os
import re

class Tarball(object):
    dsc_file = None
    def __init__(self, app_name, local_file, log):
        self.local_file = local_file
        self.log = log
        self.app_name = app_name
        self.extract_path = os.path.join(defs.get_path("tarballs"), 
                                      self.app_name)
        self.extract()
        self.find_dsc_file()

    def rename_tarball(self):
        if not self.local_file:
            return
        new_tarball = None
        if self.local_file.endswith(".tar_1.gz"):
            new_tarball = re.sub(".tar_1.gz", ".tar.gz", self.local_file)
        if self.local_file.endswith(".tgz"):
            new_tarball = re.sub(".tgz", ".tar.gz", self.local_file)
        if not new_tarball:
            return
        actions = LoggedActions()
        actions.add(Action(["mv", self.local_file, new_tarball], 
                            expected_retcode=0))
        self.log.add(logged_actions=actions)
        if not self.log.success:
            self.log.add(error="Renaming '%s' to '%s' failed." % \
                         (self.local_file, new_tarball))
        else:
            self.local_file = new_tarball

    def extract(self):
        if not self.local_file:
            return
        self.rename_tarball()
        if os.path.exists(self.extract_path):
            actions = LoggedActions()
            actions.add(Action(["rm", "-r", self.extract_path], 
                               expected_retcode=0))
        os.makedirs(self.extract_path)
        
        actions = LoggedActions()
        if self.local_file.endswith(".tar.gz"):
            cmd = ["tar", "xfz", self.local_file, "-C", self.extract_path]
        elif self.local_file.endswith(".tar"):
            cmd = ["tar", "xf", self.local_file, "-C", self.extract_path]
        elif self.local_file.endswith(".tar.bz2"):
            cmd = ["tar", "xfj", self.local_file, "-C", self.extract_path]
        elif self.local_file.endswith(".zip"):
            cmd = ["unzip", self.local_file, "-d", self.extract_path]
        else:
            self.log.add(error="No idea what to do with '%s'." % self.local_file)
            return
        actions.add(Action(cmd, expected_retcode=0))
        self.log.add(logged_actions=actions)
        if not self.log.success:
            self.log.add(error="Unpacking of %s failed." % self.local_file)
            
    def find_dsc_file(self):
        for dirpath, dirnames, filenames in os.walk(self.extract_path):
            dsc_files = filter(lambda a: a.endswith(".dsc"), filenames)
            if dsc_files:
                self.dsc_file = os.path.join(dirpath, dsc_files[0])
