from unidecode import unidecode
import datetime
import urllib2
import urllib
import socket
import time
import re
import os

QUEUES = [ 
           'https://myapps.developer.ubuntu.com/dev/arb/?state=ReviewPending',
           'https://myapps.developer.ubuntu.com/dev/arb/?state=ReviewInProgress',
           'https://myapps.developer.ubuntu.com/dev/arb/?state=QAPending',
           'https://myapps.developer.ubuntu.com/dev/arb/?state=NeedsInformation'
         ]
slug_regex = "[\w+-.]+"

def connect(url):
    try:
        socket.setdefaulttimeout(15)
        req = urllib2.Request(url)
        sock = urllib2.urlopen(req)
    except:
        return None
    return sock

def find_app_urls_in_queue(queue_url):
    sock = connect(queue_url)
    if not sock:
        return []
    html = sock.read()
    sock.close()
    regex = '<tr>\s+<td class="icon">\s+<a href="(\S+?)">\s+<img src="\S+?" />'
    app_urls = re.findall(regex, html)
    return map(lambda a: 'https://myapps.developer.ubuntu.com'+a, app_urls)

def get_last_modified(url, tarball_timestamp):
    sock = connect(url)
    if not sock:
        return (None, None)
    datetime_lm = None
    if sock.info().has_key("Last-Modified") and sock.info()["Last-Modified"]:
        lm_string = sock.info()["Last-Modified"]
        if lm_string:
            timestamp_lm = time.mktime(time.strptime(lm_string, "%a, %d %b %Y %H:%M:%S GMT"))
            datetime_lm = datetime.datetime.utcfromtimestamp(timestamp_lm)
    if tarball_timestamp and datetime_lm and datetime_lm <= tarball_timestamp:
        sock.close()
        return (None, None)
    return (sock, datetime_lm)

def find_tarball_link(html):
    regex = '<div class="detail-content">\s+<a href="(.*)">\S*?</a>'
    links = re.findall(regex, html)
    if not links:
        return ''
    return 'https://myapps.developer.ubuntu.com/'+links[0]

def find_suggested_package_name(html):
    regex = '<p>Suggested package name:</p>\s+<div class="detail-content">(.*?)</div>'
    names = re.findall(regex, html)
    if not names:
        return ''
    name = unidecode(names[0]).lower()
    name = re.sub('[^\w\s-]', '', name).strip().lower()
    name = re.sub('[-\s]+', '-', name)
    return name

def find_app_name(html):
    regex = '<p>Application name:</p>\s+<div class="detail-content">(.*?)</div>'
    names = re.findall(regex, html)
    if not names:
        return ''
    name = unidecode(names[0]).lower()
    name = re.sub('[^\w\s-]', '', name).strip().lower()
    name = re.sub('[-\s]+', '-', name)
    return name

def get_app_data(url):
    sock = connect(url)
    if not sock:
        return (None, None)
    html = sock.read()
    sock.close()
    tarball_link = find_tarball_link(html)
    app_name = find_suggested_package_name(html)
    if not app_name:
        app_name = find_app_name(html)
    return (app_name, tarball_link)

def rewrite_branch_location(location):
    location = re.sub("(http|https)://(bazaar.|code.|edge.|translations.|bugs.|translations.|answers.|)launchpad.net/", "lp:", location)
    return location

def find_branch_info(html):
    branch_regexes = [
            "lp:%s" % slug_regex,
            "lp:~%s/%s/%s" % (slug_regex, slug_regex, slug_regex),
            "(http|https)://bazaar.launchpad.net/~%s/%s/%s" % (slug_regex, slug_regex, slug_regex),
            "(http|https)://code.launchpad.net/~%s/%s/%s" % (slug_regex, slug_regex, slug_regex),
            ]
    branches = re.findall("(%s)" % "|".join(branch_regexes), html)
    branches = map(lambda a: rewrite_branch_location(a[0]), branches)
    branches = filter(lambda a: "/+archive/" not in a, branches)
    if not branches:
        return []
    return branches

def rewrite_ppa_location(location):
    location = re.sub("(http|https)://(bazaar.|code.|edge.|translations.|bugs.|translations.|answers.|)launchpad.net/", 
                      "https://launchpad.net/", location)
    location = re.sub('\.$', '', location)
    return location

def find_ppa_info(html):
    slug_regex = "[\w+-.]+"
    ppa_regexes = [
            "ppa:%s/%s" % (slug_regex, slug_regex),
            "(http|https)://(code.|edge.|translations.|bugs.|translations.|answers.|)launchpad.net/~%s/\+archive/%s" % (slug_regex, slug_regex)
            ]
    ppas = re.findall("(%s)" % "|".join(ppa_regexes), html)
    ppas = map(lambda a: rewrite_ppa_location(a[0]), ppas)
    if not ppas:
        return []
    return ppas

def retrieve_file(file_url, location):
    try:
        filename = os.path.join(location, 
                                urllib.unquote_plus(os.path.basename(file_url)))
        urllib.urlretrieve(file_url, filename)
        return filename
    except:
        return None
